@@ -10,6 +10,41 @@
 #define H_PERL 1
 #define OVERLOAD
 
+#ifdef PERL_FOR_X2P
+/*
+ * This file is being used for x2p stuff. 
+ * Above symbol is defined via -D in 'x2p/Makefile.SH'
+ * Decouple x2p stuff from some of perls more extreme eccentricities. 
+ */
+#undef MULTIPLICITY
+#undef EMBED
+#undef USE_STDIO
+#define USE_STDIO
+#endif /* PERL_FOR_X2P */
+
+/*
+ * STMT_START { statements; } STMT_END;
+ * can be used as a single statement, as in
+ * if (x) STMT_START { ... } STMT_END; else ...
+ *
+ * Trying to select a version that gives no warnings...
+ */
+#if !(defined(STMT_START) && defined(STMT_END))
+# if defined(__GNUC__) && !defined(__STRICT_ANSI__)
+#   define STMT_START	(void)(	/* gcc supports ``({ STATEMENTS; })'' */
+#   define STMT_END	)
+# else
+   /* Now which other defined()s do we need here ??? */
+#  if (VOIDFLAGS) && (defined(sun) || defined(__sun__))
+#   define STMT_START	if (1)
+#   define STMT_END	else (void)0
+#  else
+#   define STMT_START	do
+#   define STMT_END	while (0)
+#  endif
+# endif
+#endif
+
 #include "embed.h"
 
 #define VOIDUSED 1
@@ -78,13 +113,40 @@
 #   endif
 #endif
 
-#include <stdio.h>
-#ifdef USE_NEXT_CTYPE
-#include <appkit/NXCType.h>
-#else
-#include <ctype.h>
+#ifndef _TYPES_		/* If types.h defines this it's easy. */
+#   ifndef major		/* Does everyone's types.h define this? */
+#	include <sys/types.h>
+#   endif
 #endif
 
+#ifdef __cplusplus
+#  ifndef I_STDARG
+#    define I_STDARG 1
+#  endif
+#endif
+
+#ifdef I_STDARG
+#  include <stdarg.h>
+#else
+#  ifdef I_VARARGS
+#    include <varargs.h>
+#  endif
+#endif
+
+#include "perlio.h"
+
+#ifdef USE_NEXT_CTYPE
+
+#if NX_CURRENT_COMPILER_RELEASE >= 400
+#include <objc/NXCType.h>
+#else /*  NX_CURRENT_COMPILER_RELEASE < 400 */
+#include <appkit/NXCType.h>
+#endif /*  NX_CURRENT_COMPILER_RELEASE >= 400 */
+
+#else /* !USE_NEXT_CTYPE */
+#include <ctype.h>
+#endif /* USE_NEXT_CTYPE */
+
 #ifdef I_LOCALE
 #include <locale.h>
 #endif
@@ -114,13 +176,15 @@
 */
 #ifdef MYMALLOC
 #   ifdef HIDEMYMALLOC
-#	define malloc Mymalloc
-#	define realloc Myremalloc
-#	define free Myfree
+#	define malloc Perl_malloc
+#	define realloc Perl_realloc
+#	define free Perl_free
+#	define calloc Perl_calloc
 #   endif
 #   define safemalloc malloc
 #   define saferealloc realloc
 #   define safefree free
+#   define safecalloc calloc
 #endif
 
 #define MEM_SIZE Size_t
@@ -208,12 +272,6 @@
 #   endif
 #endif
 
-#ifndef _TYPES_		/* If types.h defines this it's easy. */
-#   ifndef major		/* Does everyone's types.h define this? */
-#	include <sys/types.h>
-#   endif
-#endif
-
 #ifdef I_NETINET_IN
 #   include <netinet/in.h>
 #endif
@@ -283,7 +341,7 @@
 #else
 #   define FIXSTATUS(sts)  (U_L(sts))
 #   define SHIFTSTATUS(sts) (sts)
-#   define SETERRNO(errcode,vmserrcode) {set_errno(errcode); set_vaxc_errno(vmserrcode);}
+#   define SETERRNO(errcode,vmserrcode) STMT_START {set_errno(errcode); set_vaxc_errno(vmserrcode);} STMT_END
 #endif
 
 #ifndef MSDOS
@@ -470,7 +528,7 @@
 #   define SLOPPYDIVIDE
 #endif
 
-#if defined(cray) || defined(convex) || defined (uts) || BYTEORDER > 0xffff
+#if defined(cray) || defined(convex) || BYTEORDER > 0xffff
 #   define HAS_QUAD
 #endif
 
@@ -482,7 +540,7 @@
 #   ifdef cray
 #	define Quad_t int
 #   else
-#	if defined(convex) || defined (uts)
+#	if defined(convex)
 #	    define Quad_t long long
 #	else
 #	    define Quad_t long
@@ -495,6 +553,61 @@
     typedef unsigned long UV;
 #endif
 
+/* Previously these definitions used hardcoded figures. 
+ * It is hoped these formula are more portable, although
+ * no data one way or another is presently known to me.
+ * The "PERL_" names are used because these calculated constants
+ * do not meet the ANSI requirements for LONG_MAX, etc., which
+ * need to be constants acceptable to #if - kja
+ *    define PERL_LONG_MAX        2147483647L
+ *    define PERL_LONG_MIN        (-LONG_MAX - 1)
+ *    define PERL ULONG_MAX       4294967295L
+ */
+
+#ifdef I_LIMITS  /* Needed for cast_xxx() functions below. */
+#  include <limits.h>
+#else
+#ifdef I_VALUES
+#  include <values.h>
+#endif
+#endif
+
+#ifdef LONG_MAX
+#define PERL_LONG_MAX LONG_MAX
+#else
+#  ifdef MAXLONG    /* Often used in <values.h> */
+#    define PERL_LONG_MAX MAXLONG
+#  else
+#    define PERL_LONG_MAX        ((long) ((~(unsigned long)0) >> 1))
+#  endif
+#endif
+
+#ifdef LONG_MIN
+#define PERL_LONG_MIN LONG_MIN
+#else
+#  ifdef MINLONG
+#    define PERL_LONG_MIN MINLONG
+#  else
+#    define PERL_LONG_MIN        (-LONG_MAX - ((3 & -1) == 3))
+#  endif
+#endif
+
+#ifdef ULONG_MAX
+#define PERL_ULONG_MAX ULONG_MAX
+#else
+#  ifdef MAXULONG
+#    define PERL_ULONG_MAX MAXULONG
+#  else
+#    define PERL_ULONG_MAX       (~(unsigned long)0)
+#  endif
+#endif
+
+#ifdef ULONG_MIN
+#define PERL_ULONG_MIN ULONG_MIN
+#else
+#  define ULONG_MIN	    0L
+#endif
+
 typedef MEM_SIZE STRLEN;
 
 typedef struct op OP;
@@ -519,7 +632,7 @@ typedef struct hv HV;
 typedef struct cv CV;
 typedef struct regexp REGEXP;
 typedef struct gp GP;
-typedef struct sv GV;
+typedef struct gv GV;
 typedef struct io IO;
 typedef struct context CONTEXT;
 typedef struct block BLOCK;
@@ -558,10 +671,21 @@ typedef I32 (*filter_t) _((int, SV *, int));
 # if defined(VMS)
 #   include "vmsish.h"
 # else
-#   include "unixish.h"
+#   if defined(PLAN9)
+#     include "./plan9/plan9ish.h"
+#   else
+#     include "unixish.h"
+#   endif
 # endif
 #endif
 
+#ifndef SH_PATH			/* May be a variable. */
+#   define SH_PATH BIN_SH
+#ifndef BIN_SH
+#   define BIN_SH "/bin/sh"
+#endif
+#endif
+
 #ifndef HAS_PAUSE
 #define pause() sleep((32767<<16)+32767)
 #endif
@@ -707,6 +831,9 @@ Gid_t getegid _((void));
 #endif
 
 #ifdef DEBUGGING
+#ifndef Perl_debug_log
+#define Perl_debug_log	PerlIO_stderr()
+#endif
 #define YYDEBUG 1
 #define DEB(a)     			a
 #define DEBUG(a)   if (debug)		a
@@ -717,7 +844,7 @@ Gid_t getegid _((void));
 #define DEBUG_o(a) if (debug & 16)	a
 #define DEBUG_c(a) if (debug & 32)	a
 #define DEBUG_P(a) if (debug & 64)	a
-#define DEBUG_m(a) if (debug & 128)	a
+#define DEBUG_m(a) if (curinterp && debug & 128)	a
 #define DEBUG_f(a) if (debug & 256)	a
 #define DEBUG_r(a) if (debug & 512)	a
 #define DEBUG_x(a) if (debug & 1024)	a
@@ -826,13 +953,16 @@ I32 unlnk _((char*));
 #define SCAN_TR 1
 #define SCAN_REPL 2
 
+#ifdef MYMALLOC
+# ifndef DEBUGGING_MSTATS
+#  define DEBUGGING_MSTATS
+# endif
+#endif
+
 #ifdef DEBUGGING
 # ifndef register
 #  define register
 # endif
-# ifdef MYMALLOC
-# define DEBUGGING_MSTATS
-# endif
 # define PAD_SV(po) pad_sv(po)
 #else
 # define PAD_SV(po) curpad[po]
@@ -844,9 +974,18 @@ I32 unlnk _((char*));
 
 /* global state */
 EXT PerlInterpreter *	curinterp;	/* currently running interpreter */
-#ifndef VMS  /* VMS doesn't use environ array */
+/* VMS doesn't use environ array and NeXT has problems with crt0.o globals */
+#if !defined(VMS) && !(defined(NeXT) && defined(__DYNAMIC__))
 extern char **	environ;	/* environment variables supplied via exec */
-#endif
+#else
+#  if defined(NeXT) && defined(__DYNAMIC__)
+
+#  include <mach-o/dyld.h>
+EXT char *** environ_pointer;
+#  define environ (*environ_pointer)
+#  endif
+#endif /* environ processing */
+
 EXT int		uid;		/* current real user id */
 EXT int		euid;		/* current effective user id */
 EXT int		gid;		/* current real group id */
@@ -861,6 +1000,7 @@ EXT char **	origenviron;
 EXT U32		origalen;
 EXT U32 *	profiledata;
 EXT int		maxo INIT(MAXO);/* Number of ops */
+EXT char *	osname;		/* operating system */
 
 EXT XPV*	xiv_arenaroot;	/* list of allocated xiv areas */
 EXT IV **	xiv_root;	/* free xiv list--shared by interpreters */
@@ -965,9 +1105,13 @@ EXT SV		sv_yes;
 #ifdef DOINIT
 EXT char *sig_name[] = { SIG_NAME };
 EXT int   sig_num[]  = { SIG_NUM };
+EXT SV	* psig_ptr[sizeof(sig_num)/sizeof(*sig_num)];
+EXT SV  * psig_name[sizeof(sig_num)/sizeof(*sig_num)];
 #else
 EXT char *sig_name[];
 EXT int   sig_num[];
+EXT SV  * psig_ptr[];
+EXT SV  * psig_name[];
 #endif
 
 #ifdef DOINIT
@@ -1101,7 +1245,7 @@ EXT YYSTYPE	nextval[5];	/* value of next token, if any */
 EXT I32		nexttype[5];	/* type of next token */
 EXT I32		nexttoke;
 
-EXT FILE * VOL	rsfp INIT(Nullfp);
+EXT PerlIO * VOL	rsfp INIT(Nullfp);
 EXT SV *	linestr;
 EXT char *	bufptr;
 EXT char *	oldbufptr;
@@ -1124,6 +1268,7 @@ EXT CV *	compcv;		/* currently compiling subroutine */
 EXT AV *	comppad;	/* storage for lexically scoped temporaries */
 EXT AV *	comppad_name;	/* variable names for "my" variables */
 EXT I32		comppad_name_fill;/* last "introduced" variable offset */
+EXT I32		comppad_name_floor;/* start of vars in innermost block */
 EXT I32		min_intro_pending;/* start of vars to introduce */
 EXT I32		max_intro_pending;/* end of vars to introduce */
 EXT I32		padix;		/* max used index in current "register" pad */
@@ -1150,6 +1295,7 @@ EXT U32		hints;		/* various compilation flags */
 #define HINT_BLOCK_SCOPE	0x00000100
 #define HINT_STRICT_SUBS	0x00000200
 #define HINT_STRICT_VARS	0x00000400
+#define HINT_STRICT_UNTIE	0x00000800
 
 /**************************************************************************/
 /* This regexp stuff is global since it always happens within 1 expr eval */
@@ -1174,6 +1320,9 @@ EXT char *	regtill;	/* How far we are required to go. */
 EXT U16		regflags;	/* are we folding, multilining? */
 EXT char	regprev;	/* char before regbol, \n if none */
 
+EXT bool	do_undump;	/* -u or dump seen? */
+EXT VOL U32	debug;
+
 /***********************************************/
 /* Global only to current interpreter instance */
 /***********************************************/
@@ -1207,6 +1356,7 @@ IEXT SV *	Iparsehook;
 IEXT char *	Icddir;
 IEXT bool	Iminus_c;
 IEXT char	Ipatchlevel[10];
+IEXT char **	Ilocalpatches;
 IEXT SV *	Inrs;
 IEXT char *	Isplitstr IINIT(" ");
 IEXT bool	Ipreprocess;
@@ -1223,11 +1373,9 @@ IEXT bool	Isawstudy;	/* do fbm_instr on all strings */
 IEXT bool	Isawi;		/* study must assume case insensitive */
 IEXT bool	Isawvec;
 IEXT bool	Iunsafe;
-IEXT bool	Ido_undump;		/* -u or dump seen? */
 IEXT char *	Iinplace;
 IEXT char *	Ie_tmpname;
-IEXT FILE *	Ie_fp;
-IEXT VOL U32	Idebug;
+IEXT PerlIO *	Ie_fp;
 IEXT U32	Iperldb;
 	/* This value may be raised by extensions for testing purposes */
 IEXT int	Iperl_destruct_level;	/* 0=none, 1=full, 2=full with checks */
@@ -1288,8 +1436,7 @@ IEXT HV *	Idebstash;	/* symbol table for perldb package */
 IEXT SV *	Icurstname;	/* name of current package */
 IEXT AV *	Ibeginav;	/* names of BEGIN subroutines */
 IEXT AV *	Iendav;		/* names of END subroutines */
-IEXT AV *	Ipad;		/* storage for lexically scoped temporaries */
-IEXT AV *	Ipadname;	/* variable names for "my" variables */
+IEXT HV *	Istrtab;	/* shared string table */
 
 /* memory management */
 IEXT SV **	Itmps_stack;
@@ -1335,6 +1482,7 @@ IEXT OP *	Ieval_start;
 
 /* runtime control stuff */
 IEXT COP * VOL	Icurcop IINIT(&compiling);
+IEXT COP *	Icurcopdb IINIT(NULL);
 IEXT line_t	Icopline IINIT(NOLINE);
 IEXT CONTEXT *	Icxstack;
 IEXT I32	Icxstack_ix IINIT(-1);
@@ -1343,7 +1491,7 @@ IEXT Sigjmp_buf	Itop_env;
 IEXT I32	Irunlevel;
 
 /* stack stuff */
-IEXT AV *	Istack;		/* THE STACK */
+IEXT AV *	Icurstack;		/* THE STACK */
 IEXT AV *	Imainstack;	/* the stack when nothing funny is happening */
 IEXT SV **	Imystack_base;	/* stack->array_ary */
 IEXT SV **	Imystack_sp;	/* stack pointer now */
@@ -1393,20 +1541,6 @@ struct interpreter {
 extern "C" {
 #endif
 
-#ifdef __cplusplus
-#  ifndef I_STDARG
-#    define I_STDARG 1
-#  endif
-#endif
-
-#ifdef I_STDARG
-#  include <stdarg.h>
-#else
-#  ifdef I_VARARGS
-#    include <varargs.h>
-#  endif
-#endif
-
 #include "proto.h"
 
 #ifdef EMBED
@@ -1433,8 +1567,10 @@ EXT MGVTBL vtbl_envelem =	{0,	magic_setenv,
 					0,	magic_clearenv,
 							0};
 EXT MGVTBL vtbl_sig =	{0,	0,		 0, 0, 0};
-EXT MGVTBL vtbl_sigelem =	{0,	magic_setsig,
-					0,	0,	0};
+EXT MGVTBL vtbl_sigelem =	{magic_getsig,
+					magic_setsig,
+					0,	magic_clearsig,
+							0};
 EXT MGVTBL vtbl_pack =	{0,	0,	0,	magic_wipepack,
 							0};
 EXT MGVTBL vtbl_packelem =	{magic_getpack,
@@ -1590,4 +1726,12 @@ enum {
 };
 #endif /* OVERLOAD */
 
+#if !defined(PERLIO_IS_STDIO) && defined(HAS_ATTRIBUTE)
+/* 
+ * Now we have __attribute__ out of the way 
+ * Remap printf 
+ */
+#define printf PerlIO_stdoutf
+#endif
+
 #endif /* Include guard */
