@@ -126,13 +126,13 @@ PP(pp_rv2gv)
 	GP *ogp = GvGP(sv);
 
 	SSCHECK(3);
-	SSPUSHPTR(sv);
+	SSPUSHPTR(SvREFCNT_inc(sv));
 	SSPUSHPTR(ogp);
 	SSPUSHINT(SAVEt_GP);
 
 	if (op->op_flags & OPf_SPECIAL) {
 	    GvGP(sv)->gp_refcnt++;		/* will soon be assigned */
-	    GvFLAGS(sv) |= GVf_INTRO;
+	    GvINTRO_on(sv);
 	}
 	else {
 	    GP *gp;
@@ -141,21 +141,13 @@ PP(pp_rv2gv)
 	    GvREFCNT(sv) = 1;
 	    GvSV(sv) = NEWSV(72,0);
 	    GvLINE(sv) = curcop->cop_line;
-	    GvEGV(sv) = sv;
+	    GvEGV(sv) = (GV*)sv;
 	}
     }
     SETs(sv);
     RETURN;
 }
 
-PP(pp_sv2len)
-{
-    dSP; dTARGET;
-    dPOPss;
-    PUSHi(sv_len(sv));
-    RETURN;
-}
-
 PP(pp_rv2sv)
 {
     dSP; dTOPss;
@@ -171,7 +163,7 @@ PP(pp_rv2sv)
 	}
     }
     else {
-	GV *gv = sv;
+	GV *gv = (GV*)sv;
 	char *sym;
 
 	if (SvTYPE(gv) != SVt_PVGV) {
@@ -189,24 +181,15 @@ PP(pp_rv2sv)
 	    sym = SvPV(sv, na);
 	    if (op->op_private & HINT_STRICT_REFS)
 		DIE(no_symref, sym, "a SCALAR");
-	    gv = (SV*)gv_fetchpv(sym, TRUE, SVt_PV);
+	    gv = (GV*)gv_fetchpv(sym, TRUE, SVt_PV);
 	}
 	sv = GvSV(gv);
     }
     if (op->op_flags & OPf_MOD) {
 	if (op->op_private & OPpLVAL_INTRO)
 	    sv = save_scalar((GV*)TOPs);
-	else if (op->op_private & (OPpDEREF_HV|OPpDEREF_AV)) {
-	    if (SvGMAGICAL(sv))
-		mg_get(sv);
-	    if (!SvOK(sv)) {
-		(void)SvUPGRADE(sv, SVt_RV);
-		SvRV(sv) = (op->op_private & OPpDEREF_HV ?
-			    (SV*)newHV() : (SV*)newAV());
-		SvROK_on(sv);
-		SvSETMAGIC(sv);
-	    }
-	}
+	else if (op->op_private & (OPpDEREF_HV|OPpDEREF_AV))
+	    provide_ref(op, sv);
     }
     SETs(sv);
     RETURN;
@@ -256,22 +239,42 @@ PP(pp_rv2cv)
     GV *gv;
     HV *stash;
 
-    /* We always try to add a non-existent subroutine in case of AUTOLOAD. */
-    CV *cv = sv_2cv(TOPs, &stash, &gv, TRUE);
+    /* We usually try to add a non-existent subroutine in case of AUTOLOAD. */
+    /* (But not in defined().) */
+    CV *cv = sv_2cv(TOPs, &stash, &gv, !(op->op_flags & OPf_SPECIAL));
 
+    if (!cv)
+	cv = (CV*)&sv_undef;
     SETs((SV*)cv);
     RETURN;
 }
 
+PP(pp_prototype)
+{
+    dSP;
+    CV *cv;
+    HV *stash;
+    GV *gv;
+    SV *ret;
+
+    ret = &sv_undef;
+    cv = sv_2cv(TOPs, &stash, &gv, FALSE);
+    if (cv && SvPOK(cv)) {
+	char *p = SvPVX(cv);
+	ret = sv_2mortal(newSVpv(p ? p : "", SvLEN(cv)));
+    }
+    SETs(ret);
+    RETURN;
+}
+
 PP(pp_anoncode)
 {
     dSP;
     CV* cv = (CV*)cSVOP->op_sv;
     EXTEND(SP,1);
 
-    if (SvFLAGS(cv) & SVpcv_CLONE) {
+    if (CvCLONE(cv))
 	cv = (CV*)sv_2mortal((SV*)cv_clone(cv));
-    }
 
     PUSHs((SV*)cv);
     RETURN;
@@ -328,8 +331,12 @@ PP(pp_ref)
     char *pv;
 
     sv = POPs;
+
+    if (sv && SvGMAGICAL(sv))
+	mg_get(sv);     
+
     if (!sv || !SvROK(sv))
-	RETPUSHUNDEF;
+	RETPUSHNO;
 
     sv = SvRV(sv);
     pv = sv_reftype(sv,TRUE);
@@ -355,7 +362,7 @@ PP(pp_bless)
 
 PP(pp_study)
 {
-    dSP; dTARGET;
+    dSP; dPOPss;
     register unsigned char *s;
     register I32 pos;
     register I32 ch;
@@ -364,11 +371,17 @@ PP(pp_study)
     I32 retval;
     STRLEN len;
 
-    s = (unsigned char*)(SvPV(TARG, len));
+    s = (unsigned char*)(SvPV(sv, len));
     pos = len;
-    if (lastscream)
-	SvSCREAM_off(lastscream);
-    lastscream = TARG;
+    if (sv == lastscream)
+	SvSCREAM_off(sv);
+    else {
+	if (lastscream) {
+	    SvSCREAM_off(lastscream);
+	    SvREFCNT_dec(lastscream);
+	}
+	lastscream = SvREFCNT_inc(sv);
+    }
     if (pos <= 0) {
 	retval = 0;
 	goto ret;
@@ -411,7 +424,8 @@ PP(pp_study)
 	    sfirst[fold[ch]] = pos;
     }
 
-    SvSCREAM_on(TARG);
+    SvSCREAM_on(sv);
+    sv_magic(sv, Nullsv, 'g', Nullch, 0);	/* piggyback on m//g magic */
     retval = 1;
   ret:
     XPUSHs(sv_2mortal(newSViv((I32)retval)));
@@ -481,11 +495,11 @@ PP(pp_defined)
 	RETPUSHNO;
     switch (SvTYPE(sv)) {
     case SVt_PVAV:
-	if (AvMAX(sv) >= 0)
+	if (AvMAX(sv) >= 0 || SvRMAGICAL(sv))
 	    RETPUSHYES;
 	break;
     case SVt_PVHV:
-	if (HvARRAY(sv))
+	if (HvARRAY(sv) || SvRMAGICAL(sv))
 	    RETPUSHYES;
 	break;
     case SVt_PVCV:
@@ -533,17 +547,20 @@ PP(pp_undef)
 	cv_undef((CV*)sv);
 	sub_generation++;
 	break;
+    case SVt_PVGV:
+        if (SvFAKE(sv)) {
+            sv_setsv(sv, &sv_undef);
+            break;
+        }
     default:
-	if (sv != GvSV(defgv)) {
-	    if (SvPOK(sv) && SvLEN(sv)) {
-		(void)SvOOK_off(sv);
-		Safefree(SvPVX(sv));
-		SvPV_set(sv, Nullch);
-		SvLEN_set(sv, 0);
-	    }
-	    (void)SvOK_off(sv);
-	    SvSETMAGIC(sv);
+	if (SvPOK(sv) && SvLEN(sv)) {
+	    (void)SvOOK_off(sv);
+	    Safefree(SvPVX(sv));
+	    SvPV_set(sv, Nullch);
+	    SvLEN_set(sv, 0);
 	}
+	(void)SvOK_off(sv);
+	SvSETMAGIC(sv);
     }
 
     RETPUSHUNDEF;
@@ -553,8 +570,13 @@ PP(pp_predec)
 {
     dSP;
     if (SvIOK(TOPs)) {
-	--SvIVX(TOPs);
-	SvFLAGS(TOPs) &= ~(SVf_NOK|SVf_POK|SVp_NOK|SVp_POK);
+    	if (SvIVX(TOPs) == PERL_LONG_MIN) {
+    	    sv_setnv(TOPs, (double)SvIVX(TOPs) - 1.0);
+    	}
+    	else {
+    	    --SvIVX(TOPs);
+	    SvFLAGS(TOPs) &= ~(SVf_NOK|SVf_POK|SVp_NOK|SVp_POK);
+	}
     }
     else
 	sv_dec(TOPs);
@@ -567,8 +589,13 @@ PP(pp_postinc)
     dSP; dTARGET;
     sv_setsv(TARG, TOPs);
     if (SvIOK(TOPs)) {
-	++SvIVX(TOPs);
-	SvFLAGS(TOPs) &= ~(SVf_NOK|SVf_POK|SVp_NOK|SVp_POK);
+    	if (SvIVX(TOPs) == PERL_LONG_MAX) {
+    	    sv_setnv(TOPs, (double)SvIVX(TOPs) + 1.0);
+    	}
+    	else {
+	    ++SvIVX(TOPs);
+	    SvFLAGS(TOPs) &= ~(SVf_NOK|SVf_POK|SVp_NOK|SVp_POK);
+	}
     }
     else
 	sv_inc(TOPs);
@@ -584,8 +611,13 @@ PP(pp_postdec)
     dSP; dTARGET;
     sv_setsv(TARG, TOPs);
     if (SvIOK(TOPs)) {
-	--SvIVX(TOPs);
-	SvFLAGS(TOPs) &= ~(SVf_NOK|SVf_POK|SVp_NOK|SVp_POK);
+    	if (SvIVX(TOPs) == PERL_LONG_MIN) {
+    	    sv_setnv(TOPs, (double)SvIVX(TOPs) - 1.0);
+    	}
+    	else {
+	    --SvIVX(TOPs);
+	    SvFLAGS(TOPs) &= ~(SVf_NOK|SVf_POK|SVp_NOK|SVp_POK);
+	}
     }
     else
 	sv_dec(TOPs);
@@ -885,7 +917,7 @@ PP(pp_bit_and) {
     dSP; dATARGET; tryAMAGICbin(band,opASSIGN); 
     {
       dPOPTOPssrl;
-      if (SvNIOK(left) || SvNIOK(right)) {
+      if (SvNIOKp(left) || SvNIOKp(right)) {
 	unsigned long value = U_L(SvNV(left));
 	value = value & U_L(SvNV(right));
 	SETn((double)value);
@@ -903,7 +935,7 @@ PP(pp_bit_xor)
     dSP; dATARGET; tryAMAGICbin(bxor,opASSIGN); 
     {
       dPOPTOPssrl;
-      if (SvNIOK(left) || SvNIOK(right)) {
+      if (SvNIOKp(left) || SvNIOKp(right)) {
 	unsigned long value = U_L(SvNV(left));
 	value = value ^ U_L(SvNV(right));
 	SETn((double)value);
@@ -921,7 +953,7 @@ PP(pp_bit_or)
     dSP; dATARGET; tryAMAGICbin(bor,opASSIGN); 
     {
       dPOPTOPssrl;
-      if (SvNIOK(left) || SvNIOK(right)) {
+      if (SvNIOKp(left) || SvNIOKp(right)) {
 	unsigned long value = U_L(SvNV(left));
 	value = value | U_L(SvNV(right));
 	SETn((double)value);
@@ -939,9 +971,11 @@ PP(pp_negate)
     dSP; dTARGET; tryAMAGICun(neg);
     {
 	dTOPss;
-	if (SvNIOK(sv))
+	if (SvGMAGICAL(sv))
+	    mg_get(sv);
+	if (SvNIOKp(sv))
 	    SETn(-SvNV(sv));
-	else if (SvPOK(sv)) {
+	else if (SvPOKp(sv)) {
 	    STRLEN len;
 	    char *s = SvPV(sv, len);
 	    if (isALPHA(*s) || *s == '_') {
@@ -956,6 +990,8 @@ PP(pp_negate)
 		sv_setnv(TARG, -SvNV(sv));
 	    SETTARG;
 	}
+	else
+	    SETn(-SvNV(sv));
     }
     RETURN;
 }
@@ -976,7 +1012,7 @@ PP(pp_complement)
       dTOPss;
       register I32 anum;
 
-      if (SvNIOK(sv)) {
+      if (SvNIOKp(sv)) {
 	IV iv = ~SvIV(sv);
 	if (iv < 0)
 	    SETn( (double) ~U_L(SvNV(sv)) );
@@ -1215,11 +1251,31 @@ PP(pp_srand)
 {
     dSP;
     I32 anum;
-    Time_t when;
 
     if (MAXARG < 1) {
+#ifdef VMS
+#  include <starlet.h>
+	unsigned int when[2];
+	_ckvmssts(sys$gettim(when));
+	anum = when[0] ^ when[1];
+#else
+#  if defined(I_SYS_TIME) && !defined(PLAN9)
+	struct timeval when;
+	gettimeofday(&when,(struct timezone *) 0);
+	anum = when.tv_sec ^ when.tv_usec;
+#  else
+	Time_t when;
 	(void)time(&when);
 	anum = when;
+#  endif
+#endif
+#if !defined(PLAN9) /* XXX Plan9 assembler chokes on this; fix coming soon  */
+                    /*     17-Jul-1996  bailey@genetics.upenn.edu           */
+	/* What is a good hashing algorithm here? */
+	anum ^= (  (  269 * (U32)getpid())
+	         ^ (26107 * (U32)&when)
+	         ^ (73819 * (U32)stack_sp));
+#endif
     }
     else
 	anum = POPi;
@@ -1302,28 +1358,38 @@ PP(pp_hex)
 {
     dSP; dTARGET;
     char *tmps;
+    unsigned long value;
     I32 argtype;
 
     tmps = POPp;
-    XPUSHi( scan_hex(tmps, 99, &argtype) );
+    value = scan_hex(tmps, 99, &argtype);
+    if ((IV)value >= 0)
+	XPUSHi(value);
+    else
+	XPUSHn(U_V(value));
     RETURN;
 }
 
 PP(pp_oct)
 {
     dSP; dTARGET;
-    I32 value;
+    unsigned long value;
     I32 argtype;
     char *tmps;
 
     tmps = POPp;
-    while (*tmps && (isSPACE(*tmps) || *tmps == '0'))
+    while (*tmps && isSPACE(*tmps))
+	tmps++;
+    if (*tmps == '0')
 	tmps++;
     if (*tmps == 'x')
-	value = (I32)scan_hex(++tmps, 99, &argtype);
+	value = scan_hex(++tmps, 99, &argtype);
     else
-	value = (I32)scan_oct(tmps, 99, &argtype);
-    XPUSHi(value);
+	value = scan_oct(tmps, 99, &argtype);
+    if ((IV)value >= 0)
+	XPUSHi(value);
+    else
+	XPUSHn(U_V(value));
     RETURN;
 }
 
@@ -1374,7 +1440,8 @@ PP(pp_substr)
 	    rem = len;
 	sv_setpvn(TARG, tmps, rem);
 	if (lvalue) {			/* it's an lvalue! */
-	    (void)SvPOK_only(sv);
+	    if (!SvGMAGICAL(sv))
+		(void)SvPOK_only(sv);
 	    if (SvTYPE(TARG) < SVt_PVLV) {
 		sv_upgrade(TARG, SVt_PVLV);
 		sv_magic(TARG, Nullsv, 'x', Nullch, 0);
@@ -1692,7 +1759,7 @@ PP(pp_quotemeta)
 
     if (len) {
 	(void)SvUPGRADE(TARG, SVt_PV);
-	SvGROW(TARG, len * 2);
+	SvGROW(TARG, (len * 2) + 1);
 	d = SvPVX(TARG);
 	while (len--) {
 	    if (!isALNUM(*s))
@@ -1760,18 +1827,19 @@ PP(pp_each)
 {
     dSP; dTARGET;
     HV *hash = (HV*)POPs;
-    HE *entry = hv_iternext(hash);
-    I32 i;
-    char *tmps;
+    HE *entry;
+    
+    PUTBACK;
+    entry = hv_iternext(hash);                        /* might clobber stack_sp */
+    SPAGAIN;
 
     EXTEND(SP, 2);
     if (entry) {
-	tmps = hv_iterkey(entry, &i);
-	if (!i)
-	    tmps = "";
-	PUSHs(sv_2mortal(newSVpv(tmps, i)));
+	PUSHs(hv_iterkeysv(entry));	              /* won't clobber stack_sp */
 	if (GIMME == G_ARRAY) {
-	    sv_setsv(TARG, hv_iterval(hash, entry));
+	    PUTBACK;
+	    sv_setsv(TARG, hv_iterval(hash, entry));  /* might clobber stack_sp */
+	    SPAGAIN;
 	    PUSHs(TARG);
 	}
     }
@@ -1797,14 +1865,12 @@ PP(pp_delete)
     SV *sv;
     SV *tmpsv = POPs;
     HV *hv = (HV*)POPs;
-    char *tmps;
     STRLEN len;
     if (SvTYPE(hv) != SVt_PVHV) {
 	DIE("Not a HASH reference");
     }
-    tmps = SvPV(tmpsv, len);
-    sv = hv_delete(hv, tmps, len,
-	op->op_private & OPpLEAVE_VOID ? G_DISCARD : 0);
+    sv = hv_delete_ent(hv, tmpsv,
+	(op->op_private & OPpLEAVE_VOID ? G_DISCARD : 0), 0);
     if (!sv)
 	RETPUSHUNDEF;
     PUSHs(sv);
@@ -1816,13 +1882,11 @@ PP(pp_exists)
     dSP;
     SV *tmpsv = POPs;
     HV *hv = (HV*)POPs;
-    char *tmps;
     STRLEN len;
     if (SvTYPE(hv) != SVt_PVHV) {
 	DIE("Not a HASH reference");
     }
-    tmps = SvPV(tmpsv, len);
-    if (hv_exists(hv, tmps, len))
+    if (hv_exists_ent(hv, tmpsv, 0))
 	RETPUSHYES;
     RETPUSHNO;
 }
@@ -1830,23 +1894,22 @@ PP(pp_exists)
 PP(pp_hslice)
 {
     dSP; dMARK; dORIGMARK;
-    register SV **svp;
+    register HE *he;
     register HV *hv = (HV*)POPs;
     register I32 lval = op->op_flags & OPf_MOD;
 
     if (SvTYPE(hv) == SVt_PVHV) {
 	while (++MARK <= SP) {
-	    STRLEN keylen;
-	    char *key = SvPV(*MARK, keylen);
+	    SV *keysv = *MARK;
 
-	    svp = hv_fetch(hv, key, keylen, lval);
+	    he = hv_fetch_ent(hv, keysv, lval, 0);
 	    if (lval) {
-		if (!svp || *svp == &sv_undef)
-		    DIE(no_helem, key);
+		if (!he || HeVAL(he) == &sv_undef)
+		    DIE(no_helem, SvPV(keysv, na));
 		if (op->op_private & OPpLVAL_INTRO)
-		    save_svref(svp);
+		    save_svref(&HeVAL(he));
 	    }
-	    *MARK = svp ? *svp : &sv_undef;
+	    *MARK = he ? HeVAL(he) : &sv_undef;
 	}
     }
     if (GIMME != G_ARRAY) {
@@ -1880,6 +1943,8 @@ PP(pp_lslice)
     SV **firstlelem = stack_base + POPMARK + 1;
     register SV **firstrelem = lastlelem + 1;
     I32 arybase = curcop->cop_arybase;
+    I32 lval = op->op_flags & OPf_MOD;
+    I32 is_something_there = lval;
 
     register I32 max = lastrelem - lastlelem;
     register SV **lelem;
@@ -1918,8 +1983,13 @@ PP(pp_lslice)
 	    if (ix >= max || !(*lelem = firstrelem[ix]))
 		*lelem = &sv_undef;
 	}
+	if (!is_something_there && (SvOKp(*lelem) || SvGMAGICAL(*lelem)))
+	    is_something_there = TRUE;
     }
-    SP = lastlelem;
+    if (is_something_there)
+	SP = lastlelem;
+    else
+	SP = firstlelem - 1;
     RETURN;
 }
 
@@ -1940,14 +2010,12 @@ PP(pp_anonhash)
 
     while (MARK < SP) {
 	SV* key = *++MARK;
-	char *tmps;
 	SV *val = NEWSV(46, 0);
 	if (MARK < SP)
 	    sv_setsv(val, *++MARK);
 	else
 	    warn("Odd number of elements in hash list");
-	tmps = SvPV(key,len);
-	(void)hv_store(hv,tmps,len,val,0);
+	(void)hv_store_ent(hv,key,val,0);
     }
     SP = ORIGMARK;
     XPUSHs((SV*)hv);
@@ -2788,6 +2856,8 @@ PP(pp_unpack)
 	case 'u':
 	    along = (strend - s) * 3 / 4;
 	    sv = NEWSV(42, along);
+	    if (along)
+		SvPOK_on(sv);
 	    while (s < strend && *s > ' ' && *s < 'a') {
 		I32 a, b, c, d;
 		char hunk[4];
@@ -3283,9 +3353,10 @@ PP(pp_split)
     I32 origlimit = limit;
     I32 realarray = 0;
     I32 base;
-    AV *oldstack = stack;
+    AV *oldstack = curstack;
     register REGEXP *rx = pm->op_pmregexp;
     I32 gimme = GIMME;
+    I32 oldsave = savestack_ix;
 
     if (!pm || !s)
 	DIE("panic: do_split");
@@ -3305,7 +3376,7 @@ PP(pp_split)
 	av_extend(ary,0);
 	av_clear(ary);
 	/* temporarily switch stacks */
-	SWITCHSTACK(stack, ary);
+	SWITCHSTACK(curstack, ary);
     }
     base = SP - stack_base;
     orig = s;
@@ -3313,6 +3384,11 @@ PP(pp_split)
 	while (isSPACE(*s))
 	    s++;
     }
+    if (pm->op_pmflags & (PMf_MULTILINE|PMf_SINGLELINE)) {
+	SAVEINT(multiline);
+	multiline = pm->op_pmflags & PMf_MULTILINE;
+    }
+
     if (!limit)
 	limit = maxiters + 2;
     if (pm->op_pmflags & PMf_WHITE) {
@@ -3391,7 +3467,7 @@ PP(pp_split)
     else {
 	maxiters += (strend - s) * rx->nparens;
 	while (s < strend && --limit &&
-	    regexec(rx, s, strend, orig, 1, Nullsv, TRUE) ) {
+	    pregexec(rx, s, strend, orig, 1, Nullsv, TRUE) ) {
 	    if (rx->subbase
 	      && rx->subbase != orig) {
 		m = s;
@@ -3424,6 +3500,7 @@ PP(pp_split)
 	    s = rx->endp[0];
 	}
     }
+    LEAVE_SCOPE(oldsave);
     iters = (SP - stack_base) - base;
     if (iters > maxiters)
 	DIE("Split loop");
